# Generated by Django 3.0.4 on 2020-07-22 20:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_pagarme', '0002_pagarmeitemconfig_upsell'),
    ]

    operations = [
        migrations.AddField(
            model_name='userpaymentprofile',
            name='card_id',
            field=models.CharField(blank=True, default=None, max_length=64, null=True),
        ),
    ]
